package com.pms.ocp.service.impl;

import cn.hutool.core.collection.CollectionUtil;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.github.pagehelper.PageHelper;
import com.github.pagehelper.PageInfo;
import com.pms.ocp.common.constants.CodeEnum;
import com.pms.ocp.mapper.ModelClassifyMapper;
import com.pms.ocp.model.entity.ModelType;
import com.pms.ocp.model.vo.OcpModelGroupVO;
import com.pms.ocp.model.vo.ResponseVO;
import com.pms.ocp.service.ModelClassifyService;
import com.pms.ocp.service.OcpModelGroupService;
import org.apache.commons.lang3.StringUtils;
import org.springframework.stereotype.Service;

import javax.annotation.Resource;
import java.util.*;

/**
 * @author huxiuwu
 * @version 1.0
 * @date 2022/3/8 15:20
 */
@Service
public class ModelClassifyServiceImpl implements ModelClassifyService {

    @Resource
    private ModelClassifyMapper classifyMapper;

    @Resource
    private OcpModelGroupService ocpService;

    /**
     * 模型分类-导航树
     *
     * @param searchCondition 搜索条件
     * @return List<com.pms.ocp.model.vo.OcpModelGroupVO>
     * @author huxiuwu
     * @date 2022/3/8
     **/
    @Override
    public ResponseVO<List<OcpModelGroupVO>> getModelNavigation(String searchCondition) {
        //根据是否传入搜索条件创建构造器进行查询
        List<OcpModelGroupVO> modelTypeList = this.selectByCondition(searchCondition, null);
        if (CollectionUtil.isEmpty(modelTypeList)) {
            return ResponseVO.error(CodeEnum.NO_DATA);
        }
        //对模型分类信息进行处理
        List<OcpModelGroupVO> resultList = this.build(modelTypeList);
        return ResponseVO.ok(resultList);
    }

    /**
     * 右侧子类详情列表
     *
     * @return pms.ocp.model.vo.ResponseVO
     * @author huxiuwu
     * @date 2022/3/9
     **/
    @Override
    public ResponseVO<PageInfo<OcpModelGroupVO>> subclassDetailsList(int pageSize,int pageNum,String parentGroupCode,String subclassGroupName) {
        PageHelper.startPage(pageSize,pageNum);
        List<OcpModelGroupVO> ocpModelGroupVOList = this.selectByCondition(null,null);
        LinkedList<OcpModelGroupVO> childrenList = new LinkedList<>();
        findChildren(ocpModelGroupVOList, parentGroupCode,childrenList);
        PageInfo<OcpModelGroupVO> pageInfo = new PageInfo<>(childrenList);
        return ResponseVO.ok(pageInfo);
    }

    /**
     * 修改
     *
     * @param list 修改list
     * @return pms.ocp.model.vo.ResponseVO
     * @author huxiuwu
     * @date 2022/3/9
     **/
    @Override
    public ResponseVO update(List<ModelType> list) {
        boolean isUpdateSucceed = ocpService.saveOrUpdateBatch(list);
        if (!isUpdateSucceed) {
            ResponseVO.error(CodeEnum.UP_DATA_FAIL);
        }
        return ResponseVO.ok();
    }

    /**
     * 新增
     *
     * @param list 新增分类列表
     * @return pms.ocp.model.vo.ResponseVO
     * @author huxiuwu
     * @date 2022/3/9
     **/
    @Override
    public ResponseVO insert(List<ModelType> list) {
        boolean result = ocpService.saveBatch(list);
        if (!result) {
            return ResponseVO.error(CodeEnum.INSERT_FAIL);
        }
        return ResponseVO.ok();
    }

    /**
     * 获取父节点下的所有子节点放入list中
     * @author huxiuwu
     * @date 2022/3/9
     * @param menuList
     * @param pid
     * @param childMenu
     * @return void
     **/
    private void findChildren(List<OcpModelGroupVO> menuList, String pid,List<OcpModelGroupVO> childMenu) {
        for (OcpModelGroupVO mu : menuList) {
            //遍历出父id等于参数的id，add进子节点集合
            if (pid != null && pid.equals(mu.getModelGroupPcode())) {
                //递归遍历下一级
                childMenu.add(mu);
                findChildren(menuList, mu.getModelGroupCode(), childMenu);
            }
        }

    }


    /**
     * 构建树节点
     */
    private List<OcpModelGroupVO> build(List<OcpModelGroupVO> treeNodes) {
        List<OcpModelGroupVO> result = new ArrayList<>();
        //list转map =>将每个对象的id作为key,自己作为value.
        Map<String, OcpModelGroupVO> nodeMap = new LinkedHashMap<>(treeNodes.size());
        for (OcpModelGroupVO treeNode : treeNodes) {
            nodeMap.put(treeNode.getModelGroupCode(), treeNode);
        }
        for (OcpModelGroupVO node : nodeMap.values()) {
            //得到自己的父类
            OcpModelGroupVO parent = nodeMap.get(node.getModelGroupPcode());
            if (parent != null && !(node.getModelGroupCode().equals(parent.getModelGroupCode()))) {
                if (parent.getChildrenList() == null) {
                    parent.setChildrenList(new LinkedList<>());
                }
                parent.getChildrenList().add(node);
                continue;
            }
            result.add(node);
        }
        return result;
    }

    private List<OcpModelGroupVO> selectByCondition(String... searchCondition) {
        QueryWrapper<OcpModelGroupVO> queryWrapper = new QueryWrapper<>();
        //根据是否传入搜索条件创建构造器进行查询
        queryWrapper.eq(StringUtils.isNotEmpty(searchCondition[0]), "model_group_pcode", searchCondition[1])
                .like(StringUtils.isNotEmpty(searchCondition[0]), "model_group_name", searchCondition[0]);
        return classifyMapper.selectList(queryWrapper);
    }
}
